<?php

namespace Drupal\academic_marksheet\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Render\Markup;
use Drupal\Core\Url;
use Drupal\taxonomy\Entity\Term;
use Symfony\Component\HttpFoundation\Response;

/**
 *
 */
class StudentResultController extends ControllerBase {

  /**
   *
   */
  public function studentResultPage($student_id) {
    // Fetch student details.
    $student_query = \Drupal::database()->select('student_details', 's')
      ->fields('s')
      ->condition('s.id', $student_id)
      ->execute()
      ->fetchAssoc();

    $course_term = Term::load($student_query['course']);
    $course_label = $course_term ? $course_term->getName() : 'Unknown';

    $semester_term = Term::load($student_query['sem_id']);
    $semester_label = $semester_term ? $semester_term->getName() : 'Unknown';

    $academic_year_term = Term::load($student_query['academic_year']);
    $academic_year_label = $academic_year_term ? $academic_year_term->getName() : 'Unknown';

    $marks_query = \Drupal::database()->select('students_data', 'sd')
      ->fields('sd', ['subject_id', 'marks_obtained'])
      ->condition('sd.student_id', $student_id)
      ->execute()
      ->fetchAllAssoc('subject_id');

    $subject_names = [];
    if (!empty($marks_query)) {
      $subject_ids = array_keys($marks_query);
      $subject_query = \Drupal::database()->select('faculty', 'f')
        ->fields('f', ['id', 'subject_name'])
        ->condition('f.id', $subject_ids, 'IN')
        ->execute()
        ->fetchAllAssoc('id');

      foreach ($subject_query as $subject) {
        $subject_names[$subject->id] = $subject->subject_name;
      }

      foreach ($marks_query as $subject_id => $marks) {
        $marks_query[$subject_id]->subject_name = $subject_names[$subject_id] ?? 'Unknown';
      }
    }

    $total_marks = array_sum(array_column($marks_query, 'marks_obtained'));

    // Build the result page content.
    $rows = [];
    foreach ($marks_query as $subject) {
      $rows[] = [
        'subject_name' => $subject->subject_name,
        'marks_obtained' => $subject->marks_obtained,
      ];
    }

    $content = '<div style="max-width: 800px; margin: 0 auto; padding: 20px; border: 1px solid #ddd; border-radius: 5px; background-color: #f9f9f9;">';
    $content .= '<h2 style="text-align: center;">Student Result</h2>';
    $content .= '<div style="margin-bottom: 20px;">';
    $content .= '<h3>Student Details</h3>';
    $content .= '<p><strong>Name:</strong> ' . htmlspecialchars($student_query['student_name']) . '</p>';
    $content .= '<p><strong>Roll Number:</strong> ' . htmlspecialchars($student_query['roll_no']) . '</p>';
    $content .= '<p><strong>Course:</strong> ' . htmlspecialchars($course_label) . '</p>';
    $content .= '<p><strong>Academic Year:</strong> ' . htmlspecialchars($academic_year_label) . '</p>';
    $content .= '<p><strong>Semester:</strong> ' . htmlspecialchars($semester_label) . '</p>';
    $content .= '<p><strong>Date of Birth:</strong> ' . htmlspecialchars($student_query['dob']) . '</p>';
    $content .= '<p><strong>Address:</strong> ' . htmlspecialchars($student_query['address']) . '</p>';
    $content .= '</div>';

    $content .= '<div>';
    $content .= '<h3>Marks</h3>';
    $content .= '<table style="width: 100%; border-collapse: collapse;">';
    $content .= '<thead><tr><th style="border: 1px solid #ddd; padding: 8px; text-align: left;">Subject</th><th style="border: 1px solid #ddd; padding: 8px; text-align: left;">Marks Obtained</th></tr></thead>';
    $content .= '<tbody>';

    foreach ($rows as $row) {
      $content .= '<tr><td style="border: 1px solid #ddd; padding: 8px;">' . htmlspecialchars($row['subject_name']) . '</td><td style="border: 1px solid #ddd; padding: 8px;">' . htmlspecialchars($row['marks_obtained']) . '</td></tr>';
    }

    $content .= '</tbody>';
    $content .= '<tfoot><tr><td style="border: 1px solid #ddd; padding: 8px; font-weight: bold;">Total</td><td style="border: 1px solid #ddd; padding: 8px; font-weight: bold;">' . htmlspecialchars($total_marks) . '</td></tr></tfoot>';
    $content .= '</table>';
    $content .= '</div>';

    $content .= '<div style="margin-top: 20px; text-align: center;">';
    $content .= '<a href="' . Url::fromRoute('academic_marksheet.student_result_pdf', ['student_id' => $student_id])->toString() . '" class="button" style="padding: 10px 20px;">Download PDF</a>';
    $content .= '</div>';
    $content .= '</div>';

    return [
      '#type' => 'markup',
      '#markup' => Markup::create($content),
    ];
  }

  /**
   * Generate PDF of student result.
   */
  public function studentResultPdf($student_id) {
    $student_query = \Drupal::database()->select('student_details', 's')
      ->fields('s')
      ->condition('s.id', $student_id)
      ->execute()
      ->fetchAssoc();
    // Fetch the course, semester, and academic year term labels.
    $course_term = Term::load($student_query['course']);
    $course_label = $course_term ? $course_term->getName() : 'Unknown';

    $semester_term = Term::load($student_query['sem_id']);
    $semester_label = $semester_term ? $semester_term->getName() : 'Unknown';

    $academic_year_term = Term::load($student_query['academic_year']);
    $academic_year_label = $academic_year_term ? $academic_year_term->getName() : 'Unknown';

    $marks_query = \Drupal::database()->select('students_data', 'sd')
      ->fields('sd', ['subject_id', 'marks_obtained'])
      ->condition('sd.student_id', $student_id)
      ->execute()
      ->fetchAllAssoc('subject_id');

    $subject_names = [];
    if (!empty($marks_query)) {
      $subject_ids = array_keys($marks_query);
      $subject_query = \Drupal::database()->select('faculty', 'f')
        ->fields('f', ['id', 'subject_name'])
        ->condition('f.id', $subject_ids, 'IN')
        ->execute()
        ->fetchAllAssoc('id');

      foreach ($subject_query as $subject) {
        $subject_names[$subject->id] = $subject->subject_name;
      }

      foreach ($marks_query as $subject_id => $marks) {
        $marks_query[$subject_id]->subject_name = $subject_names[$subject_id] ?? 'Unknown';
      }
    }

    // Calculate total marks.
    $total_marks = array_sum(array_column($marks_query, 'marks_obtained'));

    // Generate PDF content.
    $content = '<h2>Student Result</h2>';
    $content .= '<h3>Student Details</h3>';
    $content .= '<p><strong>Name:</strong> ' . htmlspecialchars($student_query['student_name']) . '</p>';
    $content .= '<p><strong>Roll Number:</strong> ' . htmlspecialchars($student_query['roll_no']) . '</p>';
    $content .= '<p><strong>Date of Birth:</strong> ' . htmlspecialchars($student_query['dob']) . '</p>';
    $content .= '<p><strong>Address:</strong> ' . htmlspecialchars($student_query['address']) . '</p>';
    $content .= '<p><strong>Course:</strong> ' . htmlspecialchars($course_label) . '</p>';
    $content .= '<p><strong>Academic Year:</strong> ' . htmlspecialchars($academic_year_label) . '</p>';
    $content .= '<p><strong>Semester:</strong> ' . htmlspecialchars($semester_label) . '</p>';

    $content .= '<h3>Marks</h3>';
    $content .= '<table border="1" style="width: 100%; border-collapse: collapse;">';
    $content .= '<thead><tr><th>Subject</th><th>Marks Obtained</th></tr></thead>';
    $content .= '<tbody>';

    foreach ($marks_query as $subject) {
      $content .= '<tr><td>' . htmlspecialchars($subject->subject_name) . '</td><td>' . htmlspecialchars($subject->marks_obtained) . '</td></tr>';
    }

    $content .= '</tbody>';
    $content .= '<tfoot><tr><td><strong>Total</strong></td><td><strong>' . htmlspecialchars($total_marks) . '</strong></td></tr></tfoot>';
    $content .= '</table>';

    // Generate PDF using TCPDF.
    $pdf = new \TCPDF();
    $pdf->AddPage();
    $pdf->SetFont('helvetica', '', 12);
    $pdf->writeHTML($content);
    $pdf->lastPage();

    $pdf_output = $pdf->Output('', 'S');

    return new Response($pdf_output, 200, [
      'Content-Type' => 'application/pdf',
      'Content-Disposition' => 'attachment; filename="student_result_' . $student_id . '.pdf"',
    ]);
  }

}
