<?php

namespace Drupal\academic_marksheet\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Provides a form to add faculty names, subject, subject code.
 */
class AdminDashboardForm extends FormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'admin_dashboard_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['teacher_name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Faculty Name'),
      '#required' => TRUE,
    ];

    $form['subject_name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Subject Name'),
      '#required' => TRUE,
    ];

    $form['subject_code'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Subject Code'),
      '#required' => TRUE,
    ];

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Add Teacher and Subject'),
    ];

    // Add a button to view existing data.
    $form['view_existing'] = [
      '#type' => 'button',
      '#value' => $this->t('View Existing Teachers and Subjects'),
      '#ajax' => [
        'callback' => '::viewExistingCallback',
        'wrapper' => 'teacher-list-wrapper',
        'event' => 'click',
      ],
    ];

    $form['teacher_list'] = [
      '#type' => 'markup',
      '#markup' => '<div id="teacher-list-wrapper"></div>',
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $teacher_name = $form_state->getValue('teacher_name');
    $subject_name = $form_state->getValue('subject_name');
    $subject_code = $form_state->getValue('subject_code');

    $connection = \Drupal::database();
    $connection->insert('faculty')
      ->fields([
        'teacher_name' => $teacher_name,
        'subject_name' => $subject_name,
        'subject_code' => $subject_code,
      ])
      ->execute();

    \Drupal::messenger()->addStatus($this->t('The teacher and subject have been successfully added.'));
  }

  /**
   * AJAX callback to view existing data.
   */
  public function viewExistingCallback(array &$form, FormStateInterface $form_state) {
    $build = [
      '#markup' => $this->getExistingTeachersAndSubjects(),
    ];
    return $build;
  }

  /**
   * AJAX callback to close existing data view.
   */
  public function closeExistingCallback(array &$form, FormStateInterface $form_state) {
    $build = [
      '#markup' => '<div id="teacher-list-wrapper"></div>',
    ];
    return $build;
  }

  /**
   * Get existing teachers and subjects.
   */
  protected function getExistingTeachersAndSubjects() {
    $connection = \Drupal::database();
    $query = $connection->select('faculty', 'f')
      ->fields('f', ['teacher_name', 'subject_name', 'subject_code'])
      ->execute();

    $results = $query->fetchAll();

    $output = '<table>';
    $output .= '<thead><tr><th>Faculty Name</th><th>Subject Name</th><th>Subject Code</th></tr></thead>';
    $output .= '<tbody>';

    foreach ($results as $row) {
      $output .= '<tr>';
      $output .= '<td>' . htmlspecialchars($row->teacher_name) . '</td>';
      $output .= '<td>' . htmlspecialchars($row->subject_name) . '</td>';
      $output .= '<td>' . htmlspecialchars($row->subject_code) . '</td>';
      $output .= '</tr>';
    }

    $output .= '</tbody>';
    $output .= '</table>';

    return $output;
  }

}
