<?php

namespace Drupal\academic_marksheet\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;

/**
 *
 */
class StudentUpdateForm extends FormBase {

  protected $studentId;

  /**
   *
   */
  public function getFormId() {
    return 'student_update_form';
  }

  /**
   *
   */
  public function buildForm(array $form, FormStateInterface $form_state, $student_id = NULL) {
    $this->studentId = $student_id;

    // Fetch student details.
    $student_query = \Drupal::database()->select('student_details', 's')
      ->fields('s')
      ->condition('s.id', $this->studentId['id'], '=')
      ->execute()
      ->fetchAssoc();

    $form['name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Student Name'),
      '#default_value' => $student_query['student_name'],
      '#required' => TRUE,
    ];

    $form['roll_no'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Roll Number'),
      '#default_value' => $student_query['roll_no'],
      '#required' => TRUE,
    ];

    $form['dob'] = [
      '#type' => 'date',
      '#title' => $this->t('Date of Birth'),
      '#default_value' => $student_query['dob'],
      '#required' => TRUE,
    ];

    $form['address'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Address'),
      '#default_value' => $student_query['address'],
      '#required' => TRUE,
    ];

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Update Student'),
    ];

    $form['cancel'] = [
      '#type' => 'link',
      '#title' => $this->t('Cancel'),
      '#url' => Url::fromRoute('student_listing.page'),
    ];

    return $form;
  }

  /**
   *
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $name = $form_state->getValue('name');
    $roll_no = $form_state->getValue('roll_no');
    $dob = $form_state->getValue('dob');
    $address = $form_state->getValue('address');

    \Drupal::database()->update('student_details')
      ->fields([
        'student_name' => $name,
        'roll_no' => $roll_no,
        'dob' => $dob,
        'address' => $address,
      ])
      ->condition('id', $this->studentId['id'])
      ->execute();

    \Drupal::messenger()->addStatus($this->t('Student details updated successfully.'));
    $form_state->setRedirect('student_listing.page');
  }

}
