<?php

namespace Drupal\academic_marksheet\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Provides a form to assign marks to students.
 */
class TeacherDashboardForm extends FormBase {

  /**
   *
   */
  public function getFormId() {
    return 'teacher_dashboard_form';
  }

  /**
   *
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    // Fetch all students from the student_details table.
    $students = [];
    $query = \Drupal::database()->select('student_details', 's');
    $query->fields('s', ['id', 'student_name']);
    $result = $query->execute();
    foreach ($result as $record) {
      $students[$record->id] = $record->student_name;
    }

    // Create a select dropdown for students.
    $form['student'] = [
      '#type' => 'select',
      '#title' => $this->t('Select Student'),
      '#options' => $students,
      '#required' => TRUE,
    ];

    // Fetch subjects from the faculty table.
    $subjects = [];
    $query = \Drupal::database()->select('faculty', 'f');
    $query->fields('f', ['id', 'subject_name']);
    $result = $query->execute();
    foreach ($result as $record) {
      $subjects[$record->id] = $record->subject_name;
    }

    // Create a select dropdown for subjects.
    $form['subject'] = [
      '#type' => 'select',
      '#title' => $this->t('Select Subject'),
      '#options' => $subjects,
      '#required' => TRUE,
    ];

    $form['marks_obtained'] = [
      '#type' => 'number',
      '#title' => $this->t('Marks Obtained'),
      '#required' => TRUE,
    ];

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Assign Marks'),
    ];

    return $form;
  }

  /**
   *
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // Get the submitted values.
    $student_id = $form_state->getValue('student');
    $subject_id = $form_state->getValue('subject');
    $marks_obtained = $form_state->getValue('marks_obtained');

    // Insert the data into the 'students_data' table.
    \Drupal::database()->insert('students_data')
      ->fields([
        'student_id' => $student_id,
        'subject_id' => $subject_id,
        'marks_obtained' => $marks_obtained,
      ])
      ->execute();

    \Drupal::messenger()->addStatus($this->t('Marks have been successfully assigned to the student.'));
  }

}
